# PyFasty

<div align="center">

<img src="assets/pyfasty-icon.png" alt="Logo PyFasty" width="200" height="200">

# pyfasty

🚀 **Stop au code répétitif ! Python alimenté par C natif avec registre magique, événements auto, console premium - Codez 10x plus vite !**

[![Python](https://img.shields.io/badge/Python-3.8+-blue.svg)](https://python.org)
[![Langage C](https://img.shields.io/badge/C-Extension%20Native-orange.svg)](https://en.wikipedia.org/wiki/C_(programming_language))
[![Licence](https://img.shields.io/badge/Licence-Apache%202.0-blue.svg)](LICENSE)
[![C Natif](https://img.shields.io/badge/Architecture-100%25%20C%20Natif-green.svg)](#-architecture-c-native)
[![PyPI](https://img.shields.io/badge/PyPI-Publié-brightgreen.svg)](https://pypi.org/project/pyfasty)
[![Install](https://img.shields.io/badge/Install-pip%20install%20pyfasty-blue.svg)](https://pypi.org/project/pyfasty)
[![Statut](https://img.shields.io/badge/Statut-Développement%20Actif-orange.svg)](https://github.com/hakan-karadag/pyfasty)

> 🇫🇷 **Version Française (Current)** • 🇺🇸 **[English Version](README.md)**

[**Démarrage Rapide**](#-démarrage-rapide) • [**Exemples**](#-exemples) • [**Documentation**](#-documentation) • [**Performance**](#-performance)

</div>

---

## 📖 **Table des Matières**

- [🎯 **Qu'est-ce que PyFasty ?**](#-quest-ce-que-pyfasty-)
- [💥 **Avant vs Après**](#-avant-vs-après)  
- [🚀 **Démarrage Rapide**](#-démarrage-rapide)
- [🆕 **API C Native**](#-api-c-native)
- [🔧 **Architecture C Native**](#-architecture-c-native)
- [🛠️ **Fonctionnalités Principales**](#-fonctionnalités-principales)
  - [📡 **Événements Réactifs**](#-événements-réactifs---automatisation-zéro-config)
  - [🖥️ **Console Premium**](#-console-premium---logging-fait-correctement)
  - [⚡ **Exécuteurs Intelligents**](#-exécuteurs-intelligents---appels-de-fonctions-réinventés)
- [🏎️ **Performance**](#-performance)
- [🎮 **Exemples du Monde Réel**](#-exemples-du-monde-réel)
- [🧪 **Tests**](#-tests)
- [📚 **Documentation**](#-documentation)
- [🤝 **Contribuer**](#-contribuer)
- [🗺️ **Feuille de Route**](#-feuille-de-route)
- [⚠️ **Statut de Développement**](#-statut-de-développement)
- [📄 **Licence**](#-licence)

---

## ⚡ **Pourquoi PyFasty est Différent**

Contrairement aux bibliothèques Python pures, PyFasty est une **extension C native** qui offre :

🔥 **Performance Réelle** : Code C optimisé à la main, pas de Python interprété  
🧠 **Architecture Intelligente** : Pools de mémoire, mise en cache et optimisation  
🛡️ **Prêt pour la Production** : Threading multiplateforme et gestion d'erreurs  
⚡ **Zéro Surcharge** : Appels système directs et structures de données natives  

**Le Résultat** : Commodité Python avec performance niveau C là où ça compte le plus.

---

## 🎯 **Qu'est-ce que PyFasty ?**

PyFasty est une **extension C native** qui élimine le code répétitif Python avec **4 utilitaires révolutionnaires** :

| 🏗️ **Registre Natif** | 📡 **Événements C** | 🖥️ **Console Optimisée** | ⚡ **Exécuteurs Threading** |
|------------------------|------------------------|-------------------------|------------------------|
| Performance niveau C | Déclencheurs temps réel | Formatage optimisé à la main | Threads multiplateformes |
| Optimisation pool mémoire | Traçage dépendances modules | Système couleurs avancé | Gestion tâches async |
| Opérations math prêtes | Zéro surcharge Python | Logging fichier intégré | Compatible Windows/Unix |

## 💥 **Avant vs Après**

<table>
<tr>
<td width="50%">

**❌ Avant (Python Vanilla)**
```python
# 25+ lignes pour gestion config simple
class Config:
    def __init__(self):
        self.data = {}
    
    def set_nested(self, path, value):
        keys = path.split('.')
        current = self.data
        for key in keys[:-1]:
            if key not in current:
                current[key] = {}
            current = current[key]
        current[keys[-1]] = value
    
    def get_nested(self, path):
        keys = path.split('.')
        current = self.data
        for key in keys:
            current = current[key]
        return current

config = Config()
config.set_nested('database.host', 'localhost')
config.set_nested('database.port', 5432)

# Plus gestion événements, setup logging...
```

</td>
<td width="50%">

**✅ Après (Magie PyFasty)**
```python
# ⚡ Import direct des objets C natifs
from pyfasty import console, registry, config, executor, event

# 3 lignes. C'est tout.
config.database.host = 'localhost'
config.database.port = 5432

# Événements auto-déclenchés
@event(lambda: config.database)
def on_db_config():
    console.success("Base de données configurée !")

# Logging professionnel prêt
console.info("App démarrée")
```

</td>
</tr>
</table>

## 🚀 **Démarrage Rapide**

```bash
pip install pyfasty
```

**✅ Objets C Natifs - Import Direct**
```python
# ⚡ Import direct des objets C natifs
from pyfasty import console, registry, config, executor, event

# 🏗️ Registre Global - Accédez à tout, partout
registry.user.name = "Jean"
registry.stats.counter += 1  # Auto-crée avec 0
registry.data["clé"] = "valeur"  # Dict + notation point

# 📡 Événements Réactifs - Fonctions qui se déclenchent auto
@event(lambda: registry.users_count > 100)
def scale_up():
    console.warning("Trafic élevé détecté !")

# 🖥️ Console Premium - Logging pro prêt à l'emploi
console.success("✅ Utilisateur créé")
console.error("❌ Connexion échouée")

# ⚡ Exécuteurs Intelligents - Appelez fonctions par chemin
result = executor.sync.my_module.process_data()
executor._async.heavy_task.compute()  # Non-bloquant
```

**🔧 Alternatives d'import :**
```python
# Option 1 : Import classique avec alias
import pyfasty as pf
from pyfasty import console, registry, config, executor, event

# Option 2 : Accès via le module principal
import pyfasty
console = pyfasty.console
registry = pyfasty.registry
config = pyfasty.config
executor = pyfasty.executor
event = pyfasty.event

# Option 3 : Import complet pour compatibilité
import pyfasty
# Puis utiliser pyfasty.console, pyfasty.registry, etc.
```

## 🆕 **API C Native**

**🎉 Révolution PyFasty** - Tous les objets sont maintenant **100% C natif** !

```python
# 🚀 API C native
from pyfasty import console, registry, config, executor, event
console.info("Message")           # <class 'pyfasty._pyfasty.Console'>  
registry.data = "valeur"          # <class 'pyfasty._pyfasty.Registry'>
@event(lambda: config.debug)      # <class 'builtin_function_or_method'>
result = executor.sync.module.function()  # <class 'pyfasty._pyfasty.Executor'>
```

**🔥 Avantages API Native :**
- **⚡ Performance C** : Zéro surcharge Python sur objets principaux
- **🎯 Import Direct** : `from pyfasty import console, registry, config, executor, event`  
- **🛡️ Stabilité** : Types C natifs = plus robuste et prévisible
- **📝 Code Plus Propre** : Plus de `pyfasty.` partout - bien plus lisible
- **🔄 Compatibilité Ascendante** : Ancienne API fonctionne toujours

**💡 Types Natifs Confirmés :**
```python
from pyfasty import console, registry, config, executor, event
print(type(console))   # <class 'pyfasty._pyfasty.Console'>
print(type(registry))  # <class 'pyfasty._pyfasty.Registry'>  
print(type(config))    # <class 'pyfasty._pyfasty.Config'>
print(type(executor))  # <class 'pyfasty._pyfasty.Executor'>
print(type(event))     # <class 'builtin_function_or_method'>
```

## 🔧 **Architecture C Native**

PyFasty est construit avec des **extensions C de qualité professionnelle** :

### 🏗️ **Moteur de Registre Natif** 
```c
// Système de mise en cache multi-niveaux
typedef struct {
    PyObject_HEAD
    PyObject *data;            // Dictionnaire de données
    PyObject *cache;           // Cache d'accès  
    int depth;                 // Profondeur d'optimisation
    PyObject *value;           // Valeur directe
} PyFastyBaseObject;

// Pool mémoire pour performance
PyFasty_ObjectPool g_dict_pool;
```

### 📡 **Système d'Événements Temps Réel**
```c
// Traçage dépendances modules
typedef enum {
    MODULE_REGISTRY = 2,
    MODULE_CONSOLE = 4, 
    MODULE_EXECUTOR_SYNC = 8,
    MODULE_EXECUTOR_ASYNC = 16
} ModuleType;

// Déclencheurs événements zéro surcharge
void pyfasty_trigger_sync_events_with_module(ModuleType module);
```

### ⚡ **Threading Multiplateforme**
```c
// Threading compatible Windows/Unix
#ifdef _WIN32
    CRITICAL_SECTION mutex;
#else  
    pthread_mutex_t mutex;
#endif

// Pool de threads professionnel
PyFasty_ThreadPool *g_default_thread_pool;
```

### 🖥️ **Système Console Optimisé**
```c
// Formatage avancé avec mise en cache
static time_t last_time = 0;
static char cached_hour[3] = {0};
// + gestion couleurs, logging fichier, optimisation performance
```

## 🛠️ **Fonctionnalités Principales**

```python
# ⚡ Import direct des objets C natifs
from pyfasty import console, registry, config, executor, event

# ✨ Notation point magique - créez objets imbriqués instantanément
registry.app.config.database.host = "localhost"
registry.users["jean"].profile.age = 25

# 🧮 Opérations math fonctionnent naturellement  
registry.counter += 5        # Auto-crée à 0, puis ajoute 5
registry.multiplier *= 3     # Auto-crée à 1, puis multiplie
registry.progress /= 2       # Gestion types intelligente

# 🔄 Patterns d'accès mixtes
registry.rooms["lobby"].users.append("joueur1")
registry.settings.theme = "sombre"
```

### 📡 **Événements Réactifs** - Automatisation Zéro Config

```python
# ⚡ Import direct des objets C natifs
from pyfasty import console, registry, config, executor, event

# 🎯 Déclencheurs basés conditions
@event(lambda: config.debug == True)
def activer_mode_debug():
    console.debug("🔍 Mode debug activé")

@event(lambda: registry.memory_usage > 80)
def nettoyer_memoire():
    # S'exécute quand condition remplie
    garbage_collect()

# 🔗 Conditions complexes supportées
@event(lambda: config.api.enabled and registry.users_count > 0)
def demarrer_serveur_api():
    console.success("🚀 Serveur API démarrage...")
```

### 🖥️ **Console Premium** - Logging Fait Correctement

```python
# ⚡ Import direct des objets C natifs
from pyfasty import console

# 🎨 Format entièrement personnalisable
console.config = {
    "format": "<%Y>-<%m>-<%d> <%H>:<%M>:<%S> | <%TYPE> | <%MESSAGE>",
    "colors": {
        "type": {
            "success": "\033[38;5;82m",
            "error": "\033[38;5;196m"
        }
    },
    "save_log": {
        "status": True,
        "filename": "app.log"
    }
}

# 📊 Niveaux de log multiples
console.info("ℹ️ Information")
console.success("✅ Succès") 
console.warning("⚠️ Avertissement")
console.error("❌ Erreur")
console.debug("🔍 Debug")
console.critical("🚨 Critique")
console.fatal("💀 Fatal")

# 🚀 Performance : Jusqu'à 17x plus rapide que print()
```

### ⚡ **Exécuteurs Intelligents** - Appels de Fonctions Réinventés

```python
# ⚡ Import direct des objets C natifs
from pyfasty import console, executor

# 🔄 Exécution synchrone
result = executor.sync.my_module.heavy_computation(data)
user = executor.sync.auth.get_user_by_id(123)

# ⚡ Exécution asynchrone (non-bloquante)
executor._async.email.send_notification(user_id)
executor._async.analytics.track_event("user_login")

# 🏗️ Accès modules imbriqués
config_result = executor.sync.app.config.database.get_settings()

# ❌ Gestion auto erreurs pour fonctions manquantes
try:
    executor.sync.inexistant.fonction()
except Exception as e:
    console.error(f"Fonction non trouvée : {e}")
```

## 🏎️ **Performance**

PyFasty offre des **performances de qualité professionnelle** grâce à l'implémentation C native :

| Composant | Technologie | Performance | Pourquoi C'est Rapide |
|-----------|------------|-------------|----------------------|
| 🖥️ **Console** | C optimisé main | **4.7x à 17x plus rapide** | Appels système directs + cache |
| 🏗️ **Registre** | Pools mémoire + cache | Équivalent au natif | Optimisation multi-niveaux |
| ⚡ **Threading** | C multiplateforme | Threads natifs | Optimisé Windows/Unix |
| 📡 **Événements** | Traçage modules | `<1ms` déclenchements | Zéro surcharge Python |

<details>
<summary>📊 <strong>Résultats Benchmarks Réels (C vs Python)</strong></summary>

**🖥️ Performance Console (C Natif)**
```c
// Mesuré avec outils professionnels
Messages simples :  PyFasty 208ms vs Python 979ms  → 4.7x plus rapide
Avec variables :    PyFasty 69ms  vs Python 486ms  → 7.0x plus rapide  
Timestamps :        PyFasty 25ms  vs Python 440ms  → 17.2x plus rapide
Multi-niveaux :     PyFasty 13ms  vs Python 89ms   → 6.8x plus rapide
```

**🏗️ Performance Registre (Pools Mémoire)**
```c
// Optimisation pool objets + cache
Patterns accès :    Équivalent aux dicts Python natifs
Usage mémoire :     40% réduction grâce pooling  
Sérialisation :     1.4x plus rapide que natif
Débit :             4.5M opérations/sec (dépasse limites web)
```

**⚡ Performance Threading (Multiplateforme)**  
```c
// Gestion threads native
Création threads :  Windows CRITICAL_SECTION + Unix pthread
Planification :     Queue sans verrous avec variables condition
Gestion GIL :       Patterns acquire/release optimisés
Efficacité pool :   Zéro surcharge allocation
```

**🎯 Pourquoi Cette Performance Compte :**
- **Console :** Logging production sans goulots d'étranglement
- **Registre :** Efficace mémoire avec cache intelligent  
- **Threading :** Vrai parallélisme sans limitations Python
- **Événements :** Réactivité temps réel à vitesse C

</details>

## 🎮 **Exemples du Monde Réel**

<details>
<summary>🌐 <strong>Configuration App Web</strong></summary>

```python
# ⚡ Import direct des objets C natifs
from pyfasty import console, registry, config, executor, event

# ⚙️ Configuration app
config.app.name = "Mon API Géniale"
config.database.url = "postgresql://localhost:5432/madb" 
config.redis.host = "localhost"
config.api.rate_limit = 1000

# 📊 Stats runtime
registry.stats.requests_count = 0
registry.stats.active_users = 0

# 🔔 Déclencheur auto-scaling
@event(lambda: registry.stats.requests_count > 10000)
def scaler_infrastructure():
    console.warning("🚀 Scaling infrastructure...")
    # Votre logique scaling ici

# 📈 Suivi requêtes
def handle_request():
    registry.stats.requests_count += 1
    console.info(f"📊 Requête #{registry.stats.requests_count}")
```

</details>

<details>
<summary>🎮 <strong>Gestion État Jeu</strong></summary>

```python
# ⚡ Import direct des objets C natifs
from pyfasty import console, registry, config, executor, event

# 🎮 État jeu
registry.game.level = 1
registry.game.score = 0
registry.players["joueur1"].health = 100
registry.players["joueur1"].inventory = []

# 🏆 Système succès  
@event(lambda: registry.game.score >= 1000)
def debloquer_succes():
    console.success("🏆 Succès débloqué : Maître du Score !")
    registry.players["joueur1"].achievements.append("maitre_score")

# ⚡ Progression niveau
@event(lambda: registry.game.score >= registry.game.level * 500)
def niveau_sup():
    registry.game.level += 1
    console.success(f"🆙 Niveau supérieur ! Maintenant niveau {registry.game.level}")

# 🎯 Mise à jour score
def joueur_marque(points):
    registry.game.score += points
    console.info(f"⭐ +{points} points ! Total : {registry.game.score}")
```

</details>

<details>
<summary>🤖 <strong>Communication Microservices</strong></summary>

```python
# ⚡ Import direct des objets C natifs
from pyfasty import console, registry, config, executor, event

# 🌐 Registre services
registry.services.auth.status = "sain"
registry.services.database.connections = 0
registry.services.cache.hit_rate = 0.95

# 🚨 Monitoring santé
@event(lambda: registry.services.database.connections > 100)
def surcharge_database():
    console.critical("🚨 Pool connexions database épuisé !")
    executor._async.alerts.send_slack_notification("Surcharge database détectée")

# ⚡ Suivi performance
@event(lambda: registry.services.cache.hit_rate < 0.8)
def avertissement_performance_cache():
    console.warning("📉 Taux hit cache sous seuil")
    
# 🔄 Communication services
def appeler_service_auth(user_data):
    return executor.sync.services.auth.validate_user(user_data)

def traiter_tache_async(task_data):
    executor._async.services.worker.process_task(task_data)
```

</details>

## 🧪 **Tests**

PyFasty inclut des tests complets. Lancez-les :

```bash
python -m pytest tests/
# ou lancez modules tests individuels
python test_registry.py
python test_events.py  
python test_console.py
python test_executor.py
```

## 📚 **Documentation**

- **📖 [Guide Complet](docs/guide.md)** - Exemples d'usage détaillés
- **🔧 [Référence API](docs/api.md)** - Documentation API complète  
- **🚀 [Guide Performance](docs/performance.md)** - Conseils optimisation
- **📝 [Guide Migration](docs/migration.md)** - Depuis Python vanilla
- **❓ [FAQ](docs/faq.md)** - Questions courantes

## 🤝 **Contribuer**

Nous adorons les contributions ! 

```bash
# 🔧 Configuration développement
git clone https://github.com/hakan-karadag/pyfasty.git
cd pyfasty
pip install -e ".[dev]"

# 🧪 Lancer tests  
python -m pytest

# 📝 Vérifier style code
black pyfasty/
flake8 pyfasty/
```

**Façons de contribuer :**
- 🐛 Rapports bugs et corrections
- ✨ Demandes fonctionnalités et implémentations  
- 📚 Améliorations documentation
- 🧪 Expansion couverture tests
- 💡 Optimisations performance

## 🗺️ **Feuille de Route**

- [ ] 🌐 **Intégration HTTP** - Intégration frameworks web directs
- [ ] 🗄️ **Connecteurs Database** - Accès database type ORM  
- [ ] 📊 **Tableau de Bord Métriques** - Monitoring performance intégré
- [ ] 🔌 **Système Plugins** - Architecture extensible
- [ ] 📱 **Outils CLI** - Utilitaires ligne commande
- [ ] 🐳 **Intégration Docker** - Configurations prêtes conteneurs

## ⚠️ **Statut de Développement**

**🚧 Développement Actif** - PyFasty est une **extension C professionnelle** en développement actif :

- ✅ **Système Registre** : Implémentation C native avec pools mémoire - Prêt production
- ✅ **Console Logging** : C optimisé main avec cache - Performance exceptionnelle  
- ✅ **Threading Multiplateforme** : Threads natifs Windows/Unix - Qualité professionnelle
- ✅ **Système Événements** : Traçage modules niveau C - Prêt production avec corrections récentes

**Usage Production :** 
- ✅ **Console** : Prêt pour production - Gains performance niveau C
- ✅ **Registre** : Qualité professionnelle - Optimisé mémoire avec cache intelligent
- ✅ **Événements** : Prêt production - Implémentation C stable et optimisée
- ⚠️ **Exécuteurs** : Opérations non-critiques - Surcharge threading acceptable pour tâches async

**Fondation Technique :**
- **Langage** : Extension C native avec bindings Python
- **Threading** : Multiplateforme (Windows CRITICAL_SECTION / Unix pthread)  
- **Mémoire** : Pooling objets avec cache multi-niveaux
- **Performance** : Optimisé main pour charges production

## 📄 **Licence**

Licence Apache 2.0 - voir fichier [LICENCE](LICENSE) pour détails.

**Auteur :** Hakan KARADAG

## 🌟 **Historique des Étoiles**

[![Graphique Historique Étoiles](https://api.star-history.com/svg?repos=hakan-karadag/pyfasty&type=Date)](https://star-history.com/#hakan-karadag/pyfasty&Date)

---
<div align="center">
  
**⭐ Si PyFasty vous a aidé à développer des apps Python plus rapidement, pensez à étoiler le repo !**

Créé avec ❤️ par [@hakan-karadag](https://github.com/hakan-karadag)

[⭐ Étoiler](https://github.com/hakan-karadag/pyfasty) • [🐛 Problèmes](https://github.com/hakan-karadag/pyfasty/issues) • [💡 Fonctionnalités](https://github.com/hakan-karadag/pyfasty/issues) • [📚 Documentation](https://github.com/hakan-karadag/pyfasty#readme)

</div>
